import 'dart:ui' as ui;

import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

import 'widget_ext.dart';

class ImageGridView extends StatefulWidget {
  final IndexedWidgetBuilder itemBuilder;
  final int itemCount;
  final int? maxItemCount;
  final Widget? clearIcon;
  final WidgetBuilder? addItemBuilder;
  final int? crossAxisCount;
  final double? mainAxisSpacing;
  final double? crossAxisSpacing;
  final VoidCallback? onTapAdd;
  final Function(int)? onTapDel;

  ImageGridView(
    this.itemBuilder,
    this.itemCount, {
    this.clearIcon,
    this.maxItemCount,
    this.addItemBuilder,
    this.crossAxisCount,
    this.mainAxisSpacing,
    this.crossAxisSpacing,
    this.onTapAdd,
    this.onTapDel,
  });

  @override
  _ImageGridViewState createState() => _ImageGridViewState();
}

class _ImageGridViewState extends State<ImageGridView> {
  late double deviceWidth;
  late double itemSize;

  @override
  void initState() {
    super.initState();
    deviceWidth = MediaQueryData.fromWindow(ui.window).size.width;
  }

  @override
  Widget build(BuildContext context) {
    // Widget _clean = widget.clearIcon ?? _defaultCleanIcon;
    int crossAxisCount = widget.crossAxisCount ?? _defaultCrossAxisCount;
    // Widget addItem = widget.addItemBuilder?.call(context) ?? _defaultAddItemWidget;
    double mainAxisSpacing = widget.mainAxisSpacing ?? _defaultSpacing;
    double crossAxisSpacing = widget.crossAxisSpacing ?? _defaultSpacing;
    itemSize = deviceWidth - (crossAxisCount - 1) * crossAxisSpacing;
    return GridView.builder(
      gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount: crossAxisCount, mainAxisSpacing: mainAxisSpacing, crossAxisSpacing: crossAxisSpacing),
      itemBuilder: _itemBuilder,
      itemCount: _itemCount(),
    );
  }

  Widget _itemBuilder(BuildContext context, int index) {
    if (index >= widget.itemCount) {
      var addItem = widget.addItemBuilder?.call(context) ?? _defaultAddItemWidget;
      return addItem.onTap(() => widget.onTapAdd?.call());
    }
    var item = widget.itemBuilder.call(context, index);
    return Stack(
      children: [
        SizedBox(width: itemSize, height: itemSize, child: item),
        Positioned(child: widget.clearIcon ?? _defaultCleanIcon, right: 0, top: 0).onTap(() => widget.onTapDel?.call(index)),
      ],
    );
  }

  int _itemCount() {
    if (widget.itemCount == widget.maxItemCount) {
      return widget.itemCount;
    }
    return widget.itemCount + 1;
  }

  // bool _isAddIconEnable() => widget.itemCount == widget.maxItemCount;

  Widget _defaultCleanIcon = Padding(padding: EdgeInsets.all(4), child: Icon(Icons.close, size: 16, color: Colors.red));
  Widget _defaultAddItemWidget = Container(
      child: Icon(Icons.add, size: 24, color: Colors.white),
      decoration: BoxDecoration(
        color: Colors.grey.shade200,
        borderRadius: BorderRadius.circular(8),
        border: Border.all(color: Colors.grey.shade300),
      ));
  int _defaultCrossAxisCount = 3;
  double _defaultSpacing = 10;
}
